# Project:   HyperI CI
# File:      src/hyperi_ci/languages/rust/__init__.py
# Purpose:   Rust language CI handlers
#
# License:   Proprietary — HYPERI PTY LIMITED
# Copyright: (c) 2026 HYPERI PTY LIMITED
